from sklearn.model_selection import train_test_split
from collections import Counter
import random
import numpy as np


def split_classification_data(datas, test_size=0.1):
    """对多分类任务中，按照不同的方式划分数据集
    
    Parameters
    ----------
    datas = [
                {"id": 2214, "label": "2", "sentence": "我退货申请撤销 一下吧"},
                {"id": 1850, "label": "7", "sentence": "好的  谢谢，希望进完发货"}
    ]

    test_size  默认为0.1
    数据集每个类别划分为验证集的比例

    Returns
    ------- 
    和原始数据格式一样划分好的数据集train_data, test_data
    """

    data_dicts = {}

    train_data = []
    test_data = []

    for data in datas:

        label = data["label"]

        if label not in data_dicts:
            data_dicts[label] = []
        data_dicts[label].append(data)

    for label in data_dicts:

        num = len(data_dicts[label])
        if num == 0:
            continue

        test_num = int(num * test_size)
        random.shuffle(data_dicts[label])

        test_data.extend(data_dicts[label][:test_num])
        train_data.extend(data_dicts[label][test_num:])

    return train_data, test_data


def count_label(labels):

    result = Counter(labels)
    return (result)


def analysis_res(y_true, y_pred, label2name):
    """
        y_true = [0,0,1,2,1,0,2]
        y_pred = [0,1,1,1,1,2,2]
        label2name = {"0":体育,"1":"经济","2":"文化"}
        
    """

    # 转移矩阵

    true, pred = {}, {}
    transfer = {}

    for label in label2name:
        true[label] = 0
        pred[label] = 0
        transfer[label] = {}
        for new_label in label2name:
            transfer[label][label2name[new_label]] = 0
    for y in y_true:
        if y in true:
            true[y] += 1
    for i, p in enumerate(y_pred):
        if p == y_true[i]:
            pred[p] += 1
        transfer[y_true[i]][label2name[p]] += 1
    res, res_transfer = {}, {}
    for label in label2name:
        if true[label] == 0:
            res[label2name[label]] = [0, 0, 0]
        else:
            res[label2name[label]] = [
                pred[label], true[label], pred[label] / true[label]
            ]
        res_transfer[label2name[label]] = transfer[label]
    return res, res_transfer



import json
import codecs
import numpy as np
from tqdm import tqdm
import csv
import orjson
import xlsxwriter

def write_json(filename, res, isIndent=False, isLine=False):

    if not isLine and (isinstance(res, dict) or isinstance(res, list)):
        if isIndent:
            json_str = json.dumps(res, ensure_ascii=False, indent=4)
            with open(filename, 'w', encoding='utf-8') as json_file:
                json_file.write(json_str)
        else:
            json_str = json.dumps(res, ensure_ascii=False)
            with open(filename, 'w', encoding='utf-8') as json_file:
                json_file.write(json_str)
    else:
        with codecs.open(filename, 'w', 'utf-8') as f:
            for formatted_instance in res:
                json_str = json.dumps(formatted_instance, ensure_ascii=False)
                f.write(json_str)
                f.write('\n')
        f.close()


def write_orjson(fileName, res):
    with open(fileName, 'wb') as json_file:
        json_str = orjson.dumps(res)
        json_file.write(json_str)
    json_file.close()


def write_txt(fileName, lists, model='normal'):
    """
        按行写入
    """

    if model == 'normal':
        f = open(fileName, 'w', encoding='utf8')
    else:
        f = open(fileName, 'a', encoding='utf8')
    for i in tqdm(lists):
        f.write(str(i))
        f.write('\n')
    f.close()


def write_npy(fileName, lists):
    """
        按行写入
    """

    np.save(fileName, lists)


def write_csv(filename, data):
    """
        将数组的内容写到csv中
        data格式：
            data = [
                        ['class','name','sex','height','year'],
                        [1,'xiaoming','male',168,23],
                        [2,'xiaohong','female',162,22],
                        [3,'xiaozhang','female',163,21],
                        [4,'xiaoli','male',158,21]
                    ]
    :param filename: 需要写入的csv文件路径
    :param data:数组格式的文件内容
    """
    f = open(filename, 'w', encoding='utf-8', newline='')
    writer = csv.writer(f)
    for i in tqdm(data):
        writer.writerow(i)
    f.close()

def write_excel(filename, data, sheet_name='Sheet1'):
    """
    将数据写到excel中， 默认sheet_name='Sheet1'， 可以自行设置
    支持两种数据格式：
            [[line1_cell1, line1_cell2], [line2_cell1, line2_cell2], ...]
        例如：
            [
                ['class','name','sex','height','year'],
                [1,'xiaoming','male',168,23],
                [2,'xiaohong','female',162,22],
                [3,'xiaozhang','female',163,21],
                [4,'xiaoli','male',158,21],
                ...
            ]

        
    :param filename: 需要写入的excel文件路径， 如'a.xlsx'
    :param data: 需要写入的数据
    :param sheet_name: sheet的名称， 默认为：Sheet1
    """

    workbook = xlsxwriter.Workbook(filename)  # 创建一个excel文件
    worksheet = workbook.add_worksheet(sheet_name)  # 在文件中创建一个名为TEST的sheet,不加名字默认为sheet1
    
    write_data = data

    if len(write_data) > 0:
        for i in range(len(write_data)):
            for j in range(len(write_data[i])):
                worksheet.write(i, j, write_data[i][j])

    workbook.close()



from fairies.read import (
    read,
    read_json,
    read_txt,
    read_csv,
    read_npy,
    read_orjson
)

from fairies.write import (
    write_json,
    write_txt,
    write_npy,
    write_csv,
    write_excel,
    write_orjson
)

from fairies.classification_utils import (
    split_classification_data,
    count_label,
    analysis_res
)

from fairies.nlp_utils import (
    label2id,
    is_chinese,
    find_lcs,
    long_substr,
    split_to_paragraph,
    split_to_sents,
    split_to_subsents,
    get_slide_window_text,
    get_cut_window_text
)

from fairies.nlp_jieba_utils import(
    jieba_init,
    jieba_add_words,
    jieba_cut,
    find_co_occurrence_word
)

from fairies.knowledge import(
    get_tongyin,
    get_tongxing,
    get_hanzi,
    get_stop_word
)

from fairies.nlp_clean_data import (
    removeLineFeed,
    cht_2_chs, # 繁体到简体
    chs_2_cht, # 简体到繁体
    strQ2B,
)



import re
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt


def text_len_analysis(texts):
    """
        可视化文本长度
        param texts 需要分析的文本数组
    
    """

    length = []

    for i in texts:
        length.append(len(i))

    fig = plt.figure()
    # plt.title('log')
    ax = fig.add_subplot()
    ax.hist(length, bins=10, color='blue', alpha=0.7)
    plt.show()


def label2id(labels):
    """
        return: id2label,label2id 
    """

    id2label = dict(enumerate(sorted(labels)))
    label2id = {j: i for i, j in id2label.items()}
    return id2label, label2id


def is_chinese(string):
    """
    检查整个字符串是否包含中文
    :param string: 需要检查的字符串
    :return: bool
    """
    for ch in string:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True

    return False


def find_lcs(s1, s2):
    """find the longest common subsequence between s1 ans s2"""
    """公共子串"""
    m = [[0 for i in range(len(s2) + 1)] for j in range(len(s1) + 1)]
    max_len = 0
    p = 0
    for i in range(len(s1)):
        for j in range(len(s2)):
            if s1[i] == s2[j]:
                m[i + 1][j + 1] = m[i][j] + 1
                if m[i + 1][j + 1] > max_len:
                    max_len = m[i + 1][j + 1]
                    p = i + 1
    return s1[p - max_len:p], max_len


def long_substr(data):
    """
        输入数组,输出多个字符传的公共子串
    """

    substr = ''
    if len(data) > 1 and len(data[0]) > 0:
        for i in range(len(data[0])):
            for j in range(len(data[0]) - i + 1):
                if j > len(substr) and is_substr(data[0][i:i + j], data):
                    substr = data[0][i:i + j]
    return substr


def is_substr(find, data):
    if len(data) < 1 and len(find) < 1:
        return False
    for i in range(len(data)):
        if find not in data[i]:
            return False
    return True


def split_to_paragraph(content, filter_length=(2, 1000)):
    """
        拆分成段落
    """

    content = re.sub(r"\s*", "", content)
    content = re.sub("([。！…？?!；;])", "\\1\1", content)
    sents = content.split("\1")
    sents = [_[:filter_length[1]] for _ in sents]

    res = []
    temp = ''
    for _ in sents:
        if len(temp + _) > filter_length[1]:
            res.append(temp)
            temp = _
        else:
            temp = temp + _
    if len(temp) > 0:
        res.append(temp)

    return res


def split_to_sents(content, filter_length=(2, 1000)):
    """
        拆分成句子
    """
    content = re.sub(r"\s*", "", content)
    content = re.sub("([。])", "\\1\1", content)
    sents = content.split("\1")
    sents = [_[:filter_length[1]] for _ in sents]
    return [_ for _ in sents if filter_length[0] <= len(_) <= filter_length[1]]


def split_to_subsents(content, filter_length=(2, 1000)):
    """
        拆分成子句
    """

    content = re.sub(r"\s*", "", content)
    content = re.sub("([。！…？?!；;,，])", "\\1\1", content)
    sents = content.split("\1")
    sents = [_[:filter_length[1]] for _ in sents]
    return [_ for _ in sents if filter_length[0] <= len(_) <= filter_length[1]]


def get_slide_window_text(text, maxlen, window):
    """
    使用移动窗口的方式分割text
    基于','分割文本,尽可能保留更多完整的信息,而不是根据字数强行分割
    
    param text 需要处理的文本
          maxlen 分割成文本的最大长度 需要小于1000
          window 滑动窗口长度
    """

    lists = split_to_subsents(text)
    lists_len = [len(i) for i in lists]

    # 在滑动的时候尽量保存完整的信息
    # 找到开始的位置

    start = 0
    sumTemp = 0

    cut_list = []

    for i in range(len(lists_len)):
        if sumTemp + lists_len[i] < maxlen:
            sumTemp += lists_len[i]
        else:
            if len(lists_len) > 1:
                cut_list.append([start, max(i - 1, start)])
                sumTemp = sumTemp - lists_len[start]
                start += 1
                while sumTemp > maxlen - window:
                    if start + 1 < i - 1:
                        start += 1
                        sumTemp = sumTemp - lists_len[start]
                    else:
                        start = max(i - 1, start)
                        sumTemp = lists_len[start]
                        break
                sumTemp += lists_len[i]

        if i == len(lists_len) - 1:
            cut_list.append([start, i])

    res = []

    for i in cut_list:
        new = ''
        for k in range(i[0], i[1] + 1):
            new += lists[k]
        res.append(new)

    return res


def get_cut_window_text(text, maxlen, window):
    """
    使用截断的方式分割text
    基于','分割文本,尽可能保留更多完整的信息,而不是根据字数强行分割
    
    param text 需要处理的文本
          maxlen 分割成文本的最大长度 需要小于1000
          window 滑动窗口长度
    """
    textlen = len(text)

    start = 0
    end = maxlen
    lists = []

    while end < textlen:
        lists.append([start, end + 1])
        start += window
        end += window

    if start < textlen:
        lists.append([start, textlen])

    res = []

    for i in lists:
        res.append(text[i[0]:i[1]])

    return res


#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='fairies',
    version='0.1.37',
    description=(
        'utils for nlp'
    ),
    author='liyuhao',
    author_email='1241225413@qq.com',
    license='Apache License 2.0',
    packages=find_packages(),
    url='https://github.com/hgliyuhao/fairies',
    install_requires=[
        'xlrd==1.2.0',
        'jieba',
        'xlwt',
        'xlutils',
        'matplotlib',
        'pandas',
        'orjson',
        'xlsxwriter',
        'scikit-learn'
    ],
)



import os
import csv
import json
from tqdm import tqdm
import xlrd
import xlwt
from xlutils.copy import copy
import numpy as np
import orjson
import datetime

# import chardet
# from chardet.universaldetector import UniversalDetector

# TODO
# import fairies as fa
# import csv
# from tqdm import tqdm
# import chardet
# from chardet.universaldetector import UniversalDetector

# def get_encoding(file):
#         detector = UniversalDetector()
#         with open(file,'rb') as f:
#             for line in f:
#                 detector.feed(line)
#                 if detector.done:
#                     break
#         detector.close()
#         print(detector.result)
#         # return chardet.detect(f.read(1100))['encoding']

# print(get_encoding("0325_3a_tj.csv"))




class read_data():
    def __init__(self, fileName, table_name="", isWithTitle=False):

        self.fileName = fileName
        self.table_name = table_name
        self.isWithTitle = isWithTitle
        self.res = []
        self.head = {}

        # TODO
        # 判断fileName在不在

        if fileName.endswith('.txt'):
            self.res = self.read_txt()
        elif fileName.endswith('.json'):
            self.res = self.read_json()
        elif fileName.endswith('.csv'):
            self.res = self.read_csv()
        elif fileName.endswith('.xlsx'):
            self.res = self.read_excel()
        elif fileName.endswith('.npy'):
            self.res = self.read_npy()
        else:
            raise ValueError(
                "The format of the read file can only be JSON, TXT, CSV, XLSX, NPY"
            )

    def read_txt(self):

        res = []
        with open(self.fileName, "r", encoding='utf8') as f:
            data = f.readlines()
            for d in tqdm(data):
                res.append(d)

        return res

    # def read_json(self):

    #     try:
    #         with open(self.fileName,'r', encoding='utf8') as f:
    #             json_data = json.load(f)
    #     except:
    #         json_data = []
    #         with open(self.fileName,encoding='utf-8') as f:
    #             for line in tqdm(f):
    #                 json_data.append(json.loads(line))
    #     return json_data

    def read_json(self):

        try:
            with open(self.fileName, 'rb') as f:
                json_data = orjson.loads(f.read())
            return json_data
        except:

            try:
                with open(self.fileName, 'r', encoding='utf8') as f:
                    json_data = json.load(f)
            except:
                json_data = []
                with open(self.fileName, encoding='utf-8') as f:
                    for line in tqdm(f):
                        json_data.append(json.loads(line))
            return json_data

    def read_excel(self):

        res = []

        data = xlrd.open_workbook(self.fileName)
        table_name = self.table_name

        if table_name != "":
            table = data.sheet_by_name(table_name)
        else:
            table = data.sheets()[0]

        rowNum = table.nrows
        colNum = table.ncols

        if self.isWithTitle:

            # TODO
            pass
            # title_lists = []
            # for j in range(colNum):
            #     title_lists.append(table.cell(0,j).value)

        else:
            for i in range(rowNum):
                row_data = []
                for j in range(colNum):
                    row_data.append(table.cell(i, j).value)
                res.append(row_data)

        return res

    def read_csv(self):
        res = []
        with open(self.fileName, encoding='utf-8') as f:
            reader = csv.reader(f)
            for l in tqdm(reader):
                res.append(l)
        return res

    def read_npy(self):

        dict_load = np.load(self.fileName, allow_pickle=True)
        return dict_load.tolist()


def read(fileName,table_name=""):

    if table_name == "":
        return read_data(fileName).res
    else:
        return read_data(fileName,table_name).res


def read_json(filename):

    try:
        with open(filename, 'r', encoding='utf8') as f:
            json_data = json.load(f)
        return json_data
    except:
        json_data = []
        with open(filename, encoding='utf-8') as f:
            for line in tqdm(f):
                json_data.append(json.loads(line))
        return json_data


def read_txt(fileName):
    """
        读取txt文件
    """

    res = []
    with open(fileName, "r", encoding='utf8') as f:
        data = f.readlines()
        for d in tqdm(data):
            res.append(d)

    return res


def read_csv(fileName):
    res = []
    with open(fileName, encoding='utf-8') as f:
        reader = csv.reader(f)
        for l in tqdm(reader):
            res.append(l)
    return res


def read_npy(fileName):

    dict_load = np.load(fileName, allow_pickle=True)
    return dict_load.tolist()


def read_orjson(fileName):

    with open(fileName, 'rb') as f:
        json_data = orjson.loads(f.read())
    return json_data


import re
from fairies.langconv import *


def removeLineFeed(text):
    """去除换行 tab键"""
    k = text.replace('\r', '').replace('\n', '').replace('\t', ' ')
    return k


def cht_2_chs(line):
    """转换繁体到简体"""
    line = Converter('zh-hans').convert(line)
    line.encode('utf-8')
    return line


# 转换简体到繁体
def chs_2_cht(sentence):
    """转换简体到繁体"""
    sentence = Converter('zh-hant').convert(sentence)
    return sentence


def strQ2B(ustring):
    """全角转半角"""
    rstring = ""
    for uchar in ustring:
        inside_code = ord(uchar)
        if inside_code == 12288:  #全角空格直接转换
            inside_code = 32
        elif (inside_code >= 65281 and inside_code <= 65374):  #全角字符（除空格）根据关系转化
            inside_code -= 65248

        rstring += chr(inside_code)
    return rstring
